@@ -1,18 +1,24 @@
 package App::rdapper;
+use App::rdapper::l10n;
+use Encode qw(decode);
+use File::ShareDir qw(:ALL);
 use Getopt::Long qw(GetOptionsFromArray :config pass_through);
 use JSON;
-use List::Util qw(any min max);
+use List::Util qw(any min max uniq);
 use Net::ASN;
 use Net::DNS::Domain;
+use Net::IDN::PP;
 use Net::IP;
 use Net::RDAP::EPPStatusMap;
-use Net::RDAP 0.35;
+use Net::RDAP 0.41;
 use Pod::Usage;
+use POSIX qw(setlocale LC_ALL);
 use Term::ANSIColor;
 use Term::Size;
 use Text::Wrap;
 use URI;
 use constant {
+    GETTEXT_DOMAIN  => q{rdapper},
     # see RFC 6350, Section 6.3.1.
     'ADR_STREET'    => 2,
     'ADR_CITY'      => 3,
@@ -22,17 +28,25 @@ use constant {
     'INDENT'        => '  ',
     'IANA_BASE_URL' => 'https://rdap.iana.org/',
 };
-use vars qw($VERSION);
+use locale;
+use vars qw($VERSION $LH);
 use strict;
 
-$VERSION = '1.10';
+$VERSION = '1.23';
+
+$LH = App::rdapper::l10n->get_handle;
+
+$LH->die_for_lookup_failures(1);
+$LH->bindtextdomain(GETTEXT_DOMAIN, $ENV{RDAPPER_LOCALE_DIR} || module_dir(__PACKAGE__));
+$LH->textdomain(GETTEXT_DOMAIN);
 
 #
-# global arg variables (note: nopager is now ignored)
+# global arg variables
 #
 my (
     $type, $object, $help, $short, $bypass, $auth, $nopager, $raw, $both,
-    $registrar, $nocolor, $reverse, $version, $search, $debug, $registry
+    $registrar, $nocolor, $reverse, $version, $search, $debug, $registry,
+    $strings
 );
 
 #
@@ -45,7 +59,7 @@ my %opts = (
     'short'         => \$short,
     'bypass-cache'  => \$bypass,
     'auth:s'        => \$auth,
-    'nopager'       => \$nopager,
+    'nopager'       => \$nopager, # ignored
     'raw'           => \$raw,
     'both'          => \$both,
     'registrar'     => \$registrar,
@@ -57,10 +71,12 @@ my %opts = (
     'debug'         => \$debug,
     'autnum'        => sub { $type = 'autnum' },
     'domain'        => sub { $type = 'domain' },
+    'nameserver'    => sub { $type = 'nameserver' },
     'entity'        => sub { $type = 'entity' },
     'ip'            => sub { $type = 'ip' },
     'tld'           => sub { $type = 'tld' },
     'url'           => sub { $type = 'url' },
+    'strings'       => \$strings,
 );
 
 my $funcs = {
@@ -72,10 +88,85 @@ my $funcs = {
     'help'       => sub { 1 }, # help only contains generic properties
 };
 
+my $OBJECT_TYPE_DISPLAY_NAMES = {
+    'ip network' => _('IP Network'),
+    'autnum'     => _('Autonomous System'),
+    'domain'     => _('Domain Name'),
+    'entity'     => _('Entity'),
+    'nameserver' => _('Nameserver'),
+    'help'       => _('Help'),
+};
+
 my @ROLE_DISPLAY_NAMES_ORDER = qw(registrant administrative technical billing
     abuse registrar reseller sponsor proxy notifications noc);
 
-my %ROLE_DISPLAY_NAMES = ('noc' => 'NOC');
+my %ROLE_DISPLAY_NAMES = (
+    'registrant'        => _('Registrant'),
+    'technical'         => _('Technical'),
+    'administrative'    => _('Administrative'),
+    'abuse'             => _('Abuse'),
+    'billing'           => _('Billing'),
+    'registrar'         => _('Registrar'),
+    'reseller'          => _('Reseller'),
+    'sponsor'           => _('Sponsor'),
+    'proxy'             => _('Proxy'),
+    'notifications'     => _('Notifications'),
+    'noc'               => _('NOC'),
+);
+
+my %EVENT_DISPLAY_NAMES = (
+    'registration'                  => _('Registration'),
+    'reregistration'                => _('Reregistration'),
+    'last changed'                  => _('Last Changed'),
+    'expiration'                    => _('Expiration'),
+    'deletion'                      => _('Deletion'),
+    'reinstantiation'               => _('Reinstantiation'),
+    'transfer'                      => _('Transfer'),
+    'locked'                        => _('Locked'),
+    'unlocked'                      => _('Unlocked'),
+    'last update of RDAP database'  => _('Last update of RDAP database'),
+    'registrar expiration'          => _('Registrar expiration'),
+    'enum validation expiration'    => _('ENUM validation expiration'),
+);
+
+my %STATUS_DISPLAY_NAMES = (
+    'validated'                     => _('validated'),
+    'renew prohibited'              => _('renew prohibited'),
+    'update prohibited'             => _('update prohibited'),
+    'transfer prohibited'           => _('transfer prohibited'),
+    'delete prohibited'             => _('delete prohibited'),
+    'proxy'                         => _('proxy'),
+    'private'                       => _('private'),
+    'removed'                       => _('removed'),
+    'obscured'                      => _('obscured'),
+    'associated'                    => _('associated'),
+    'active'                        => _('active'),
+    'inactive'                      => _('inactive'),
+    'locked'                        => _('locked'),
+    'pending create'                => _('pending create'),
+    'pending renew'                 => _('pending renew'),
+    'pending transfer'              => _('pending transfer'),
+    'pending update'                => _('pending update'),
+    'pending delete'                => _('pending delete'),
+    'add period'                    => _('add period'),
+    'auto renew period'             => _('auto renew period'),
+    'client delete prohibited'      => _('client delete prohibited'),
+    'client hold'                   => _('client hold'),
+    'client renew prohibited'       => _('client renew prohibited'),
+    'client transfer prohibited'    => _('client transfer prohibited'),
+    'client update prohibited'      => _('client update prohibited'),
+    'pending restore'               => _('pending restore'),
+    'redemption period'             => _('redemption period'),
+    'renew period'                  => _('renew period'),
+    'server delete prohibited'      => _('server delete prohibited'),
+    'server renew prohibited'       => _('server renew prohibited'),
+    'server transfer prohibited'    => _('server transfer prohibited'),
+    'server update prohibited'      => _('server update prohibited'),
+    'server hold'                   => _('server hold'),
+    'transfer period'               => _('transfer period'),
+    'administrative'                => _('administrative'),
+    'reserved'                      => _('reserved'),
+);
 
 my @EVENTS = (
     'registration',
@@ -97,26 +188,56 @@ for (my $i = 0 ; $i < scalar(@EVENTS) ; $i++) {
     $EVENT_DISPLAY_ORDER{$EVENTS[$i]} = $i;
 }
 
+my %NOTICE_REMARK_TYPE = (
+    'result set truncated due to authorization'         => _('result set truncated due to authorization'),
+    'result set truncated due to excessive load'        => _('result set truncated due to excessive load'),
+    'result set truncated due to unexplainable reasons' => _('result set truncated due to unexplainable reasons'),
+    'object truncated due to authorization'             => _('object truncated due to authorization'),
+    'object truncated due to excessive load'            => _('object truncated due to excessive load'),
+    'object truncated due to unexplainable reasons'     => _('object truncated due to unexplainable reasons'),
+    'object redacted due to authorization'              => _('object redacted due to authorization'),
+);
+
+my %VCARD_KIND_DISPLAY_NAME = (
+    'individual'    => _('individual'),
+    'group'         => _('group'),
+    'org'           => _('org'),
+    'location'      => _('location'),
+    'application'   => _('application'),
+    'device'        => _('device'),
+);
+
 my @VCARD_DISPLAY_ORDER = qw(SOURCE KIND FN TITLE ROLE ORG ADR GEO EMAIL CONTACT-URI SOCIALPROFILE TEL IMPP URL CATEGORIES NOTE);
 my %VCARD_NODE_NAMES = (
-    FN              => 'Name',
-    ORG             => 'Organization',
-    TEL             => 'Phone',
-    EMAIL           => 'Email',
-    IMPP            => 'Messaging',
-    URL             => 'Website',
-    SOCIALPROFILE   => 'Profile',
-    'CONTACT-URI'   => 'Contact Link',
-    GEO             => 'Location',
+    ADR             => _('Address'),
+    CATEGORIES      => _('Categories'),
+    'CONTACT-URI'   => _('Contact Link'),
+    EMAIL           => _('Email'),
+    FN              => _('Name'),
+    GEO             => _('Location'),
+    IMPP            => _('Messaging'),
+    KIND            => _('Kind'),
+    NOTE            => _('Note'),
+    ORG             => _('Organization'),
+    ROLE            => _('Role'),
+    SOCIALPROFILE   => _('Profile'),
+    SOURCE          => _('Source'),
+    TEL             => _('Phone'),
+    TITLE           => _('Title'),
+    URL             => _('Website'),
+);
+
+my %PUBLIC_ID_DISPLAY_NAME = (
+    'IANA Registrar ID' => _('IANA Registrar ID'),
 );
 
 my @ADR_DISPLAY_ORDER = (ADR_STREET, ADR_CITY, ADR_SP, ADR_PC, ADR_CC);
 my %ADR_DISPLAY_NAMES = (
-    &ADR_STREET => 'Street',
-    &ADR_CITY   => 'City',
-    &ADR_SP     => 'State/Province',
-    &ADR_PC     => 'Postal Code',
-    &ADR_CC     => 'Country',
+    &ADR_STREET => _('Street'),
+    &ADR_CITY   => _('City'),
+    &ADR_SP     => _('State/Province'),
+    &ADR_PC     => _('Postal Code'),
+    &ADR_CC     => _('Country'),
 );
 
 my $json = JSON->new->utf8->canonical->pretty->convert_blessed;
@@ -135,27 +256,47 @@ $Text::Wrap::huge       = 'overflow';
 sub main {
     my $package = shift;
 
+    my $rcfile = sprintf(q{%s/.rdapper}, $ENV{HOME});
+    if (-e $rcfile && open(my $fh, $rcfile)) {
+        push(@_, map { chomp ; $_ } $fh->getlines);
+        $fh->close;
+    }
+
     GetOptionsFromArray(\@_, %opts) || $package->show_usage;
 
+    #
+    # this undocumented behaviour is used to export all the translateable
+    # strings in this file.
+    #
+    export_strings() if ($strings);
+
     $ENV{NET_RDAP_UA_DEBUG} = 1 if ($debug);
 
+    my $lang = $LH->language_tag || q{en};
+    $lang =~ s/_/-/g;
+
     $rdap = Net::RDAP->new(
-        'use_cache' => !$bypass,
-        'cache_ttl' => 300,
+        'use_cache'         => !$bypass,
+        'cache_ttl'         => 300,
+        'accept_language'   => $lang,
     );
 
     $package->show_version if ($version);
 
     if ($registry && $registrar) {
-        $package->error("Cannot specify both --registry and --registrar, use one or the other.");
+        $package->error(_("cannot specify both --registry and --registrar, use one or the other."));
 
     } elsif ($registry && $both) {
-        $package->error("Cannot specify both --registry and --both, use one or the other.");
+        $package->error(_("cannot specify both --registry and --both, use one or the other."));
 
     }
 
     $registrar ||= $both;
 
+    if (!$registry && !$both) {
+        $registrar = 1;
+    }
+
     $object = shift(@_) if (!$object);
 
     $package->show_usage if ($help || length($object) < 1);
@@ -179,6 +320,7 @@ sub main {
 
     } else {
         $package->lookup($rdap, $object, $type, %args);
+
     }
 }
 
@@ -190,7 +332,7 @@ sub lookup {
     if ('ip' eq $type) {
         my $ip = Net::IP->new($object);
 
-        $package->error("Invalid IP address '$object'") unless ($ip);
+        $package->error(_("invalid IP address '[_1]'", $object)) unless ($ip);
 
         $response = $rdap->ip($ip, %args);
 
@@ -203,10 +345,10 @@ sub lookup {
         $response = $rdap->autnum(Net::ASN->new($asn), %args);
 
     } elsif ('domain' eq $type) {
-        $response = $rdap->domain(Net::DNS::Domain->new($object), %args);
+        $response = $rdap->domain(Net::DNS::Domain->new($package->encode_idn($object)), %args);
 
     } elsif ('nameserver' eq $type) {
-        my $url = Net::RDAP::Registry->get_url(Net::DNS::Domain->new($object));
+        my $url = Net::RDAP::Registry->get_url(Net::DNS::Domain->new($package->encode_idn($object)));
 
         #
         # munge path
@@ -221,7 +363,7 @@ sub lookup {
         $response = $rdap->entity($object, %args);
 
     } elsif ('tld' eq $type) {
-        $response = $rdap->fetch(URI->new(IANA_BASE_URL.'domain/'.$object), %args);
+        $response = $rdap->fetch(URI->new(IANA_BASE_URL.'domain/'.$package->encode_idn($object)), %args);
 
     } elsif ('url' eq $type) {
         my $uri = URI->new($object);
@@ -234,7 +376,7 @@ sub lookup {
         $response = $rdap->fetch($uri, %args);
 
     } else {
-        $package->error("Unable to handle type '$type'");
+        $package->error(_("Unable to handle type '[_1]'", $type));
 
     }
 
@@ -264,7 +406,7 @@ sub search {
         $package->domain_search($rdap, $object, %args);
 
     } else {
-        $package->error('Current unable to do searches for %s objects.', $type);
+        $package->error(_("current unable to do searches for '[_1]' objects.", $type));
 
     }
 }
@@ -361,23 +503,25 @@ sub display {
             $registrar = undef;
 
             if (!$link) {
-                $package->warning('No registrar link found, displaying the registry record...');
-                return $package->display($object, $indent);
+                $package->display($object, $indent);
+
+            } else {
+                my $result = $rdap->fetch($link);
+
+                if ($result->isa('Net::RDAP::Error')) {
+                    $package->display($result, $indent, 1);
+
+                    $package->warning(_('Unable to retrieve registrar record, displaying the registry record...'));
+                    $package->display($object, $indent);
+
+                } else {
+                    $package->display($object, $indent, 1) if ($both);
+
+                    $package->display($result, $indent);
+
+                }
             }
 
-            my $result = $rdap->fetch($link);
-
-            if ($result->isa('Net::RDAP::Error')) {
-                return $package->display($result, $indent, 1);
-
-                $package->warning('Unable to retrieve registrar record, displaying the registry record...');
-                return $package->display($object, $indent);
-            }
-
-            $package->display($object, $indent, 1) if ($both);
-
-            $package->display($result, $indent);
-
         } else {
             if ($raw) {
                 $out->print($json->encode($object));
@@ -396,37 +540,27 @@ sub display {
 sub display_object {
     my ($package, $object, $indent) = @_;
 
-    $package->error("JSON response does not include the 'objectClassName' properties") unless ($object->class);
-    $package->error(sprintf("Unknown object type '%s'", $object->class)) unless ($funcs->{$object->class});
+    $package->error(_("object does not include the 'objectClassName' properties")) unless ($object->class);
+    $package->error(_("unknown object type '[_1]'", $object->class)) unless ($funcs->{$object->class});
 
     #
     # generic properties
     #
-    $package->print_kv('Object type', $object->class, $indent) if ($indent < 1);
-    $package->print_kv('URL', u($object->self->href), $indent) if ($indent < 1 && $object->self);
+    $package->print_kv(_('Object type'), $OBJECT_TYPE_DISPLAY_NAMES->{$object->class}, $indent) if ($indent < 1);
+    $package->print_kv(_('URL'), u($object->self->href), $indent) if ($indent < 1 && $object->self);
 
     if ($object->can('name')) {
         my $name = $object->name;
+        $name = $name->name if ($name->can('name'));
 
-        if ($name) {
-            my $xname;
+        my $xname = $object->can('unicodeName') ? $object->unicodeName || $name : $name;
 
-            if ('Net::DNS::Domain' eq ref($name)) {
-                $xname = $name->xname;
-                $name  = $name->name;
+        if ($xname ne $name) {
+            $package->print_kv(_('Name'), sprintf('%s (%s)', uc($xname), uc($name)));
 
-            } else {
-                $xname = $name;
+        } else {
+            $package->print_kv(_('Name'), uc($name));
 
-            }
-
-            if ($xname ne $name) {
-                $package->print_kv('Name', sprintf('%s (%s)', uc($xname), uc($name)));
-
-            } else {
-                $package->print_kv('Name', uc($name));
-
-            }
         }
     }
 
@@ -461,12 +595,12 @@ sub display_object {
 
         my @fields = $object->redactions;
         if (scalar(@fields) > 0) {
-            $package->print_kv('Redacted Fields', '', $indent);
+            $package->print_kv(_('Redacted Fields'), '', $indent);
             foreach my $field (@fields) {
                 $out->print(wrap(
                     (INDENT x ($indent + 1)),
                     (INDENT x ($indent + 2)),
-                    sprintf("%s %s (reason: %s)\n", b('*'), $field->name, $field->reason)
+                    sprintf("%s %s\n", b('*'), _("[_1] (reason: [_2])", $field->name, $field->reason))
                 ));
             }
         }
@@ -478,60 +612,63 @@ sub display_object {
 sub print_ip {
     my ($package, $ip, $indent) = @_;
 
-    $package->print_kv('Handle',    $ip->handle, $indent)               if ($ip->handle);
-    $package->print_kv('Version',   $ip->version, $indent)              if ($ip->version);
-    $package->print_kv('Domain',    u($ip->domain->as_string), $indent) if ($ip->domain);
-    $package->print_kv('Type',      $ip->type, $indent)                 if ($ip->type);
-    $package->print_kv('Country',   $ip->country, $indent)              if ($ip->country);
-    $package->print_kv('Parent',    $ip->parentHandle, $indent)         if ($ip->parentHandle);
-    $package->print_kv('Range',     $ip->range->prefix, $indent)        if ($ip->range);
+    $package->print_kv(_('Handle'),     $ip->handle, $indent)               if ($ip->handle);
+    $package->print_kv(_('Version'),    $ip->version, $indent)              if ($ip->version);
+    $package->print_kv(_('Domain'),     u($ip->domain->as_string), $indent) if ($ip->domain);
+    $package->print_kv(_('Type'),       $ip->type, $indent)                 if ($ip->type);
+    $package->print_kv(_('Country'),    $ip->country, $indent)              if ($ip->country);
+    $package->print_kv(_('Parent'),     $ip->parentHandle, $indent)         if ($ip->parentHandle);
+    $package->print_kv(_('Range'),      $ip->range->prefix, $indent)        if ($ip->range);
 
     foreach my $cidr ($ip->cidrs) {
-        $package->print_kv('CIDR', $cidr->prefix, $indent);
+        $package->print_kv(_('CIDR'), $cidr->prefix, $indent);
     }
 }
 
 sub print_asn {
     my ($package, $asn, $indent) = @_;
 
-    $package->print_kv('Handle',    $asn->handle, $indent) if ($asn->handle);
-    $package->print_kv('Range',     sprintf('%u - %u', $asn->start, $asn->end), $indent) if ($asn->start > 0 && $asn->end > 0 && $asn->end > $asn->start);
-    $package->print_kv('Type',      $asn->type, $indent) if ($asn->type);
+    $package->print_kv(_('Handle'), $asn->handle, $indent) if ($asn->handle);
+    $package->print_kv(_('Range'),  sprintf('%u - %u', $asn->start, $asn->end), $indent) if ($asn->start > 0 && $asn->end > 0 && $asn->end > $asn->start);
+    $package->print_kv(_('Type'),   $asn->type, $indent) if ($asn->type);
 }
 
 sub print_domain {
     my ($package, $domain, $indent) = @_;
 
-    $package->print_kv('Handle', $domain->handle, $indent) if ($domain->handle);
+    $package->print_kv(_('Handle'), $domain->handle, $indent) if ($domain->handle);
 
     foreach my $ns (sort { lc($a->name->name) cmp lc($b->name->name) } $domain->nameservers) {
         if ($short) {
-            $package->print_kv('Nameserver', uc($ns->name->name) . ' ' . join(' ', map { $_->short } $ns->addresses), $indent);
+            $package->print_kv(_('Nameserver'), uc($ns->name->name) . ' ' . join(' ', map { $_->short } $ns->addresses), $indent);
 
         } else {
-            $package->print_kv('Nameserver', uc($ns->name->name), $indent);
+            $package->print_kv(_('Nameserver'), uc($ns->name->name), $indent);
             $package->print_nameserver($ns, 1+$indent);
         }
     }
 
     foreach my $ds ($domain->ds) {
-        $package->print_kv('DS Record', $ds->plain, $indent);
+        $package->print_kv(_('DS Record'), $ds->plain, $indent);
     }
 
     foreach my $key ($domain->keys) {
-        $package->print_kv('DNSKEY Record', $key->plain, $indent);
+        $package->print_kv(_('DNSKEY Record'), $key->plain, $indent);
     }
 
     $package->display_artRecord($domain->{'artRecord_record'}, $indent) if ($domain->{'artRecord_record'});
     $package->display_platform_nameservers($domain->{'platformNS_nameservers'}, $indent) if ($domain->{'platformNS_nameservers'});
 
-    $package->print_kv('Registration Type', $domain->{'regType_regType'}) if ($domain->{'regType_regType'});
+    $package->print_kv(_('Registration Type'), $domain->{'regType_regType'}) if ($domain->{'regType_regType'});
+
+    my @types = $domain->dns_ttl_types;
+    $package->display_ttl_values($domain, $indent) if (scalar(@types) > 0);
 }
 
 sub display_artRecord {
     my ($package, $records, $indent) = @_;
 
-    $package->print_kv('Art Record', undef, $indent);
+    $package->print_kv(_('Art Record'), undef, $indent);
 
     foreach my $record (@{$records}) {
         $package->print_kv($record->{'name'}, $record->{'value'}, 1+$indent);
@@ -542,17 +679,17 @@ sub display_platform_nameservers {
     my ($package, $nameservers, $indent) = @_;
 
     foreach my $ns (@{$nameservers}) {
-        $package->print_kv('Platform Nameserver', uc(Net::RDAP::Object::Nameserver->new($ns)->name->name), $indent);
+        $package->print_kv(_('Platform Nameserver'), uc(Net::RDAP::Object::Nameserver->new($ns)->name->name), $indent);
     }
 }
 
 sub print_entity {
     my ($package, $entity, $indent) = @_;
 
-    $package->print_kv('Handle', $entity->handle, $indent) if ($entity->handle && $indent < 1);
+    $package->print_kv(_('Handle'), $entity->handle, $indent) if ($entity->handle && $indent < 1);
 
     foreach my $id ($entity->ids) {
-        $package->print_kv($id->type, $id->identifier, $indent);
+        $package->print_kv($PUBLIC_ID_DISPLAY_NAME{$id->type} || $id->type, $id->identifier, $indent);
     }
 
     my $jcard = $entity->jcard;
@@ -579,20 +716,24 @@ sub print_jcard_property {
 
     } else {
         my $label = $VCARD_NODE_NAMES{uc($property->type)} || ucfirst(lc($property->type));
+        my $value = $property->value;
 
-        if ('TEL' eq uc($property->type)) {
+        if ('KIND' eq uc($property->type)) {
+            $value = $VCARD_KIND_DISPLAY_NAME{$value} || $value;
+
+        } elsif ('TEL' eq uc($property->type)) {
             if (any { 'fax' eq lc($_) } @{$property->param('type')}) {
-                $label = 'Fax';
+                $label = _('Fax');
 
             } else {
-                $label = 'Phone';
+                $label = _('Phone');
 
             }
         }
 
         $package->print_kv(
             $label,
-            $property->may_be_uri ? u($property->value) : $property->value,
+            $property->may_be_uri ? u($value) : $value,
             $indent
         );
     }
@@ -601,7 +742,7 @@ sub print_jcard_property {
 sub print_jcard_adr {
     my ($package, $property, $indent) = @_;
 
-    $package->print_kv('Address', '', $indent);
+    $package->print_kv(_('Address'), '', $indent);
 
     if ($property->param('label')) {
         $out->print(wrap(
@@ -627,29 +768,34 @@ sub print_jcard_adr {
     }
 
     if ($property->param('cc')) {
-        $package->print_kv('Country', $property->param('cc'), $indent+1);
+        $package->print_kv(_('Country'), $property->param('cc'), $indent+1);
     }
 }
 
 sub print_nameserver {
     my ($package, $nameserver, $indent) = @_;
 
-    $package->print_kv('Handle', $nameserver->handle, $indent) if ($nameserver->handle);
+    $package->print_kv(_('Handle'), $nameserver->handle, $indent) if ($nameserver->handle);
 
     foreach my $ip ($nameserver->addresses) {
-        $package->print_kv('IP Address', $ip->short, $indent);
+        $package->print_kv(_('IP Address'), $ip->short, $indent);
     }
+
+    my @types = $nameserver->dns_ttl_types;
+    $package->display_ttl_values($nameserver, $indent) if (scalar(@types) > 0);
 }
 
 sub print_events {
     my ($package, $object, $indent) = @_;
 
     foreach my $event (sort { $EVENT_DISPLAY_ORDER{$a->action} - $EVENT_DISPLAY_ORDER{$b->action} } $object->events) {
+        my $action = $EVENT_DISPLAY_NAMES{$event->action} || ucfirst($event->action);
+
         if ($event->actor) {
-            $package->print_kv(ucfirst($event->action), sprintf('%s (by %s)', scalar($event->date), $event->actor), $indent);
+            $package->print_kv($action, _('[_1] (by [_2])', scalar($event->date), $event->actor), $indent);
 
         } else {
-            $package->print_kv(ucfirst($event->action), scalar($event->date).$event->date_tz, $indent);
+            $package->print_kv($action, scalar($event->date).$event->date_tz, $indent);
 
         }
     }
@@ -659,12 +805,20 @@ sub print_status {
     my ($package, $object, $indent, $is_domain) = @_;
 
     foreach my $status ($object->status) {
+
         my $epp = rdap2epp($status);
         if ($epp && $is_domain && !$short) {
-            $package->print_kv('Status', sprintf('%s (EPP: %s, %s)', $status, $epp, u(sprintf('https://icann.org/epp#%s', $epp))), $indent);
+            my $friendly = $STATUS_DISPLAY_NAMES{$status} || $status;
+            if ($friendly ne $status) {
+                $package->print_kv(_('Status'), _('[_1] ([_2], EPP: [_3], [_4])', $friendly, $status, $epp, u(sprintf('https://icann.org/epp#%s', $epp))), $indent);
+
+            } else {
+                $package->print_kv(_('Status'), _('[_1] (EPP: [_2], [_3])', $status, $epp, u(sprintf('https://icann.org/epp#%s', $epp))), $indent);
+
+            }
 
         } else {
-            $package->print_kv('Status', $status, $indent);
+            $package->print_kv(_('Status'), $status, $indent);
 
         }
     }
@@ -684,7 +838,7 @@ sub print_entities {
             if (any { $role eq $_ } $entity->roles) {
                 $seen{$i} = 1;
 
-                my $rstring = join(', ', map { sprintf('%s Contact', $ROLE_DISPLAY_NAMES{$_} || ucfirst($_)) } $entity->roles);
+                my $rstring = join(', ', map { _('[_1] Contact', $ROLE_DISPLAY_NAMES{$_} || ucfirst($_)) } $entity->roles);
 
                 if ($entity->handle && 'not applicable' ne $entity->handle && 'HANDLE REDACTED FOR PRIVACY' ne $entity->handle) {
                     $package->print_kv($rstring, $entity->handle, $indent);
@@ -705,13 +859,13 @@ sub print_entities {
 sub print_remark_or_notice {
     my ($package, $thing, $indent) = @_;
 
-    my $type = ($thing->isa('Net::RDAP::Notice') ? 'Notice' : 'Remark');
+    my $type = ($thing->isa('Net::RDAP::Notice') ? _('Notice') : _('Remark'));
 
     if (1 == scalar($thing->description)) {
         $package->print_kv($thing->title || $type, ($thing->description)[0], $indent);
 
     } else {
-        $package->print_kv($thing->title || $type, , '', $indent);
+        $package->print_kv($thing->title || $NOTICE_REMARK_TYPE{$thing->type} || $type, , '', $indent);
 
         $out->print(fill(
             (INDENT x (1+$indent)),
@@ -735,6 +889,20 @@ sub print_link {
     );
 }
 
+sub display_ttl_values {
+    my ($package, $object, $indent) = @_;
+
+    $package->print_kv(_('DNS TTL Values'), '', $indent);
+
+    foreach my $type ($object->dns_ttl_types) {
+        $package->print_kv($type, _('[_1]s', $object->dns_ttl($type)), 1+$indent);
+    }
+
+    foreach my $remark ($object->dns_ttl_remarks) {
+        $package->print_remark_or_notice($remark, 1+$indent);
+    }
+}
+
 sub print_kv {
     my ($package, $name, $value, $indent) = @_;
 
@@ -748,26 +916,26 @@ sub print_kv {
 sub debug {
     my ($package, $fmt, @params) = @_;
     if ($debug) {
-        my $str = sprintf("Debug: $fmt", @params);
+        my $str = sprintf(_("Debug: [_1]", $fmt), @params);
         $err->say(colourise([qw(magenta)], $str));
     }
 }
 
 sub info {
     my ($package, $fmt, @params) = @_;
-    my $str = sprintf("Info: $fmt", @params);
+    my $str = sprintf(_("Info: [_1]", $fmt), @params);
     $err->say(colourise([qw(cyan)], $str));
 }
 
 sub warning {
     my ($package, $fmt, @params) = @_;
-    my $str = sprintf("Warning: $fmt", @params);
+    my $str = sprintf(_("Warning: [_1]", $fmt), @params);
     $err->say(colourise([qw(yellow)], $str));
 }
 
 sub error {
     my ($package, $fmt, @params) = @_;
-    my $str = sprintf("Error: $fmt", @params);
+    my $str = sprintf(_("Error: [_1]", $fmt), @params);
     $err->say(colourise([qw(red)], $str));
     exit 1;
 }
@@ -786,6 +954,56 @@ sub colourise {
 
 sub u { colourise([qw(underline)], shift) }
 sub b { colourise([qw(bold)], shift) }
+sub _ { decode($LH->encoding, $LH->maketext(@_)) }
+
+#
+# this function uses PPI to parse this file, extract the messages passed to _()
+# and prints a .po file on STDOUT.
+#
+sub export_strings {
+    eval {
+        require PPI;
+
+        my $doc = PPI::Document->new(__FILE__);
+        $doc->prune(q{PPI::Token::Comment});
+        $doc->prune(q{PPI::Token::Whitespace});
+
+        my @msgs;
+
+        my @nodes = @{$doc->find(sub { 1 })};
+        for (my $i = 0 ; $i < scalar(@nodes) ; $i++) {
+            my $node = $nodes[$i];
+
+            if ($node->isa(q{PPI::Token::Magic}) && q{_} eq $node->content) {
+                my $next = $nodes[$i+1];
+
+                if ($next->isa(q{PPI::Structure::List})) {
+                    my $msg = ($next->tokens)[1];
+
+                    if (!$msg->isa(q{PPI::Token::Quote})) {
+                        die(sprintf(
+                            "%s: first argument to _() must be a string literal",
+                            $msg->content,
+                        ));
+                    }
+
+                    push(@msgs, $msg->string);
+                }
+            }
+        }
+
+        foreach my $msg (uniq(@msgs)) {
+            printf("msgid \"%s\"\nmsgstr \"\"\n\n", $msg);
+        }
+    };
+
+    exit;
+}
+
+sub encode_idn {
+    my ($package, $name) = @_;
+    return Net::IDN::PP->encode($name);
+}
 
 1;
 
@@ -795,25 +1013,7 @@ __END__
 
 =head1 NAME
 
-App::rdapper - a simple console-based L<RDAP|https://about.rdap.org> client.
-
-=head1 INSTALLATION
-
-To install, run:
-
-    cpanm --sudo App::rdapper
-
-=head1 RUNNING VIA DOCKER
-
-The L<git repository|https://github.com/gbxyz/rdapper> contains a
-C<Dockerfile> that can be used to build an image on your local system.
-
-Alternatively, you can pull the L<image from Docker
-Hub|https://hub.docker.com/r/gbxyz/rdapper>:
-
-    $ docker pull gbxyz/rdapper
-
-    $ docker run -it gbxyz/rdapper --help
+App::rdapper - a command-line L<RDAP|https://about.rdap.org> client.
 
 =head1 SYNOPSIS
 
@@ -835,10 +1035,8 @@ Examples:
 
 =head1 DESCRIPTION
 
-C<rdapper> is a simple RDAP client. It uses L<Net::RDAP> to retrieve data about
-internet resources (domain names, IP addresses, and autonymous systems) and
-outputs the information in a human-readable format. If you want to consume this
-data in your own program you should use L<Net::RDAP> directly.
+C<rdapper> retrieves data about internet resources (domain names, IP addresses,
+and autonymous systems) and outputs the information in a human-readable format.
 
 =head1 OPTIONS
 
@@ -876,14 +1074,16 @@ The RDAP server of the parent domain's registry will be queried.
 
 =over
 
-=item * C<--registry> - display the registry record only (the default).
+=item * C<--registry> - display the registry record only. This was the default
+behaviour prior to v1.12.
 
 =item * C<--registrar> - follow referral to the registrar's RDAP record (if
-any) which will be displayed instead of the registry record. Cannot be used with
-C<--registry>.
+any) which will be displayed instead of the registry record. If no registrar
+link can be found, the registry record will be displayed. This option cannot be
+used with C<--registry>. As of v1.12, this is the default behaviour.
 
 =item * C<--both> - display both the registry and (if any) registrar RDAP
-records (implies C<--registrar>).
+records.
 
 =item * C<--reverse> - if you provide an IP address or CIDR prefix, then this
 option causes C<rdapper> to display the record of the corresponding
@@ -935,8 +1135,8 @@ you aren't expecting them to.
 
 =head1 RDAP Search
 
-Some RDAP servers support the ability to perform simple substring searches.
-You can use the C<--search> option to enable this functionality.
+Some RDAP servers support the ability to perform simple substring searches. You
+can use the C<--search> option to enable this functionality.
 
 When the C<--search> option is used, C<OBJECT> will be used as a search term.
 If it contains no dots (e.g. C<exampl*>), then C<rdapper> will send a search
@@ -949,11 +1149,50 @@ printed to C<STDOUT>.
 
 As of writing, search is only available for domain names.
 
+=head1 INTERNATIONALIZATION & LOCALIZATION
+
+C<rdapper> is internationalized. When performing RDAP queries, it will provide
+the user's preferred language in the `Accept-Language` header, however very few
+RDAP servers currently provide localized responses. Its output will also be
+translated (where translations are available).
+
+If you would like rdapper to support your preferred language, please see the
+advice on contributing translations in L<App::rdapper:l10n>
+
+=head1 CONFIGURATION FILE
+
+If C<~/.rdapper> exists, then its contents will be appended to the list of
+arguments the program runs with. So if it contained
+
+    --short
+    --bypass-cache
+
+Then running C<rdapper EXAMPLE.COM> will behave like
+C<rdapper EXAMPLE.COM --short --bypass-cache>.
+
+=head1 INSTALLATION
+
+To install, run:
+
+    cpanm --sudo App::rdapper
+
+=head1 RUNNING VIA DOCKER
+
+The L<git repository|https://github.com/gbxyz/rdapper> contains a
+C<Dockerfile> that can be used to build an image on your local system.
+
+Alternatively, you can pull the L<image from Docker
+Hub|https://hub.docker.com/r/gbxyz/rdapper>:
+
+    $ docker pull gbxyz/rdapper
+
+    $ docker run -it gbxyz/rdapper --help
+
 =head1 COPYRIGHT & LICENSE
 
 Copyright (c) 2012-2023 CentralNic Ltd.
 
-Copyright (c) 2023-2025 Gavin Brown.
+Copyright (c) 2023-2026 Gavin Brown.
 
 All rights reserved. This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
